@@ -1,5 +1,5 @@
 ###########################################################################/**
-# @set "class=TabularTextFile"
+# @set "class=GenericTabularFile"
 # @RdocMethod writeColumnsToFiles
 #
 # @title "Read each column from a data file and exports it to a separate file"
@@ -43,7 +43,7 @@
 # @keyword IO
 # @keyword programming
 #*/###########################################################################
-setMethodS3("writeColumnsToFiles", "TabularTextFile", function(this, destPath, filenameFmt="%s.txt", tags=NULL, columnName=NULL, header=NULL, ..., verbose=FALSE) {
+setMethodS3("writeColumnsToFiles", "GenericTabularFile", function(this, destPath, filenameFmt="%s.txt", tags=NULL, columnName=NULL, header=NULL, ..., verbose=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Local function
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -115,7 +115,7 @@ setMethodS3("writeColumnsToFiles", "TabularTextFile", function(this, destPath, f
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   pathnames <- c();
 
-  colClassPatterns <- "character";
+  colClasses <- "character";
   for (cc in seq_along(columnNames)) {
     columnName <- columnNames[cc];
     verbose && enter(verbose, sprintf("Column #%d ('%s') of %d", 
@@ -128,8 +128,8 @@ setMethodS3("writeColumnsToFiles", "TabularTextFile", function(this, destPath, f
 
     # Check if file already exists
     if (!isFile(pathname)) {
-      names(colClassPatterns) <- sprintf("^%s$", columnName);
-      values <- readDataFrame(this, colClassPatterns=colClassPatterns);
+      names(colClasses) <- sprintf("^%s$", columnName);
+      values <- readDataFrame(this, colClasses=colClasses);
       values <- trim(values[[1]]);
       df <- data.frame(dummy=values, stringsAsFactors=FALSE);
       if (is.null(hdrColumnName)) {
@@ -174,6 +174,9 @@ setMethodS3("writeColumnsToFiles", "TabularTextFile", function(this, destPath, f
 
 ############################################################################
 # HISTORY:
+# 2012-12-03
+# o Generalized writeColumnsToFiles() to GenericTabularFile.  Used to
+#   be only for TabularTextFile.
 # 2011-02-18
 # o ROBUSTNESS: Now writeColumnsToFiles() for TabularTextFile writes
 #   files atomically, which should minimize the risk for generating
